summaryrefslogtreecommitdiff
Side-by-side diff
Diffstat (more/less context) (ignore whitespace changes)
-rw-r--r--noncore/tools/clock/analogclock.cpp112
-rw-r--r--noncore/tools/clock/analogclock.h3
2 files changed, 64 insertions, 51 deletions
diff --git a/noncore/tools/clock/analogclock.h b/noncore/tools/clock/analogclock.h
index 3aa035e..2287888 100644
--- a/noncore/tools/clock/analogclock.h
+++ b/noncore/tools/clock/analogclock.h
@@ -19,31 +19,32 @@
**********************************************************************/
#ifndef ANALOGCLOCK_H
#define ANALOGCLOCK_H
#include <qdatetime.h>
#include <qframe.h>
class AnalogClock : public QFrame
{
Q_OBJECT
public:
AnalogClock( QWidget *parent=0, const char *name=0 );
+ ~AnalogClock();
QSizePolicy sizePolicy() const;
void display( const QTime& time );
protected:
void drawContents( QPainter *p );
void drawHand( QPainter *p, QPoint, QPoint );
private:
QPoint rotate( QPoint center, QPoint p, int angle );
QTime currTime;
QTime prevTime;
- bool clear;
+ QPixmap *_pixmap;
};
#endif
diff --git a/noncore/tools/clock/analogclock.cpp b/noncore/tools/clock/analogclock.cpp
index c5f0155..16ecc5c 100644
--- a/noncore/tools/clock/analogclock.cpp
+++ b/noncore/tools/clock/analogclock.cpp
@@ -11,124 +11,139 @@
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "analogclock.h"
+#include <qtopia/global.h>
+
#include <qlayout.h>
#include <qpainter.h>
-#include <qtopia/global.h>
+#include <qpixmap.h>
+
#include <math.h>
const double deg2rad = 0.017453292519943295769; // pi/180
AnalogClock::AnalogClock( QWidget *parent, const char *name )
- : QFrame( parent, name ), clear(false)
+ : QFrame( parent, name )
{
setMinimumSize(50,50);
+
+ /* initial the buffer pixmap */
+ QRect r = contentsRect();
+ _pixmap = new QPixmap( r.width(), r.height() );
+}
+
+AnalogClock::~AnalogClock() {
+ delete _pixmap;
}
QSizePolicy AnalogClock::sizePolicy() const
{
return QSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
}
-void AnalogClock::drawContents( QPainter *p )
+void AnalogClock::drawContents( QPainter * )
{
-#if !defined(NO_DEBUG)
- static bool first = true;
- if ( first ) {
-// QTOPIA_PROFILE("first paint event");
- first = false;
- }
-#endif
+ /* no need to draw... */
+ if ( !isVisible() )
+ return;
+
QRect r = contentsRect();
if ( r.width() < r.height() ) {
r.setY( (r.height() - r.width())/2 );
r.setHeight( r.width() );
}
+ /* resize the buffer */
+ if ( r.width() != _pixmap->width() ||
+ r.height() != _pixmap->height() )
+ _pixmap->resize( r.width(), r.height() );
+
+ QPainter p;
+ p.begin( _pixmap );
+
+
QPoint center( r.x() + r.width() / 2, r.y() + r.height() / 2 );
const int w_tick = r.width()/300+1;
const int w_sec = r.width()/400+1;
const int w_hour = r.width()/80+1;
QPoint l1( r.x() + r.width() / 2, r.y() + 2 );
QPoint l2( r.x() + r.width() / 2, r.y() + 8 );
QPoint h1( r.x() + r.width() / 2, r.y() + r.height() / 4 );
QPoint h2( r.x() + r.width() / 2, r.y() + r.height() / 2 );
QPoint m1( r.x() + r.width() / 2, r.y() + r.height() / 9 );
QPoint m2( r.x() + r.width() / 2, r.y() + r.height() / 2 );
QPoint s1( r.x() + r.width() / 2, r.y() + 8 );
QPoint s2( r.x() + r.width() / 2, r.y() + r.height() / 2 );
- QColor color( clear ? backgroundColor() : black );
- QTime time = clear ? prevTime : currTime;
+ /* fill the pixmap */
+ _pixmap->fill( this, 0, 0 );
- if ( clear && prevTime.secsTo(currTime) > 1 ) {
- p->eraseRect( rect() );
- return;
- }
- if ( !clear ) {
- // draw ticks
- p->setPen( QPen( color, w_tick ) );
- for ( int i = 0; i < 12; i++ )
- p->drawLine( rotate( center, l1, i * 30 ), rotate( center, l2, i * 30 ) );
- }
+ // draw ticks
+ QColor color = black;
+ p.setPen( QPen( color, w_tick ) );
+ for ( int i = 0; i < 12; i++ )
+ p.drawLine( rotate( center, l1, i * 30 ), rotate( center, l2, i * 30 ) );
- if ( !clear || prevTime.minute() != currTime.minute() ||
- prevTime.hour() != currTime.hour() ) {
- // draw hour pointer
- h1 = rotate( center, h1, 30 * ( time.hour() % 12 ) + time.minute() / 2 );
- h2 = rotate( center, h2, 30 * ( time.hour() % 12 ) + time.minute() / 2 );
- p->setPen( color );
- p->setBrush( color );
- drawHand( p, h1, h2 );
- }
- if ( !clear || prevTime.minute() != currTime.minute() ) {
- // draw minute pointer
- m1 = rotate( center, m1, time.minute() * 6 );
- m2 = rotate( center, m2, time.minute() * 6 );
- p->setPen( color );
- p->setBrush( color );
- drawHand( p, m1, m2 );
- }
+ // draw hour pointer
+ h1 = rotate( center, h1, 30 * ( currTime.hour() % 12 ) + currTime.minute() / 2 );
+ h2 = rotate( center, h2, 30 * ( currTime.hour() % 12 ) + currTime.minute() / 2 );
+ p.setPen( color );
+ p.setBrush( color );
+ drawHand( &p, h1, h2 );
+
+
+ // draw minute pointer
+ m1 = rotate( center, m1, currTime.minute() * 6 );
+ m2 = rotate( center, m2, currTime.minute() * 6 );
+ p.setPen( color );
+ p.setBrush( color );
+ drawHand( &p, m1, m2 );
// draw second pointer
- s1 = rotate( center, s1, time.second() * 6 );
- s2 = rotate( center, s2, time.second() * 6 );
- p->setPen( QPen( color, w_sec ) );
- p->drawLine( s1, s2 );
+ s1 = rotate( center, s1, currTime.second() * 6 );
+ s2 = rotate( center, s2, currTime.second() * 6 );
+ p.setPen( QPen( color, w_sec ) );
+ p.drawLine( s1, s2 );
// cap
- p->setBrush(color);
- p->drawEllipse( center.x()-w_hour/2, center.y()-w_hour/2, w_hour, w_hour );
+ p.setBrush(color);
+ p.drawEllipse( center.x()-w_hour/2, center.y()-w_hour/2, w_hour, w_hour );
+
+ prevTime = currTime;
+
+ p.end();
- if ( !clear )
- prevTime = currTime;
+ p.begin( this );
+ p.drawPixmap( 0, 0, *_pixmap );
+
+ /* leave */
}
// Dijkstra's bisection algorithm to find the square root as an integer.
static uint int_sqrt(uint n)
{
if ( n >= UINT_MAX>>2 ) // n must be in the range 0...UINT_MAX/2-1
return 2*int_sqrt( n/4 );
uint h, p= 0, q= 1, r= n;
while ( q <= n )
q <<= 2;
while ( q != 1 ) {
@@ -177,27 +192,24 @@ void AnalogClock::drawHand( QPainter *p, QPoint p1, QPoint p2 )
QPointArray pa(4);
pa[0] = p1;
pa[1] = QPoint( p2.x()+iy, p2.y()-nix );
pa[2] = QPoint( p2.x()-niy, p2.y()+ix );
pa[3] = p1;
p->drawPolygon( pa );
}
void AnalogClock::display( const QTime& t )
{
currTime = t;
- clear = true;
- repaint( false );
- clear = false;
repaint( false );
}
QPoint AnalogClock::rotate( QPoint c, QPoint p, int a )
{
double angle = deg2rad * ( - a + 180 );
double nx = c.x() - ( p.x() - c.x() ) * cos( angle ) -
( p.y() - c.y() ) * sin( angle );
double ny = c.y() - ( p.y() - c.y() ) * cos( angle ) +
( p.x() - c.x() ) * sin( angle );
return QPoint( int(nx), int(ny) );
}